@@ -3,12 +3,13 @@
 from __future__ import print_function
 
 import csv
+import pytest
 
 from numpy import nan
 import numpy as np
 
 from pandas.compat import (lmap, range, lrange, StringIO, u)
-from pandas.parser import CParserError
+from pandas.errors import ParserError
 from pandas import (DataFrame, Index, Series, MultiIndex, Timestamp,
                     date_range, read_csv, compat, to_datetime)
 import pandas as pd
@@ -16,9 +17,8 @@ import pandas as pd
 from pandas.util.testing import (assert_almost_equal,
                                  assert_series_equal,
                                  assert_frame_equal,
-                                 ensure_clean,
-                                 makeCustomDataframe as mkdf,
-                                 assertRaisesRegexp, slow)
+                                 ensure_clean, slow,
+                                 makeCustomDataframe as mkdf)
 import pandas.util.testing as tm
 
 from pandas.tests.frame.common import TestData
@@ -29,9 +29,7 @@ MIXED_INT_DTYPES = ['uint8', 'uint16', 'uint32', 'uint64', 'int8', 'int16',
                     'int32', 'int64']
 
 
-class TestDataFrameToCSV(tm.TestCase, TestData):
-
-    _multiprocess_can_split_ = True
+class TestDataFrameToCSV(TestData):
 
     def test_to_csv_from_csv1(self):
 
@@ -96,8 +94,8 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 
             assert_frame_equal(xp, rs)
 
-            self.assertRaises(ValueError, self.frame2.to_csv, path,
-                              header=['AA', 'X'])
+            pytest.raises(ValueError, self.frame2.to_csv, path,
+                          header=['AA', 'X'])
 
     def test_to_csv_from_csv3(self):
 
@@ -435,13 +433,13 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             assert_frame_equal(df, result)
 
     def test_to_csv_with_mix_columns(self):
-        # GH11637, incorrect output when a mix of integer and string column
+        # gh-11637: incorrect output when a mix of integer and string column
         # names passed as columns parameter in to_csv
 
         df = DataFrame({0: ['a', 'b', 'c'],
                         1: ['aa', 'bb', 'cc']})
         df['test'] = 'txt'
-        self.assertEqual(df.to_csv(), df.to_csv(columns=[0, 1, 'test']))
+        assert df.to_csv() == df.to_csv(columns=[0, 1, 'test'])
 
     def test_to_csv_headers(self):
         # GH6186, the presence or absence of `index` incorrectly
@@ -477,7 +475,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 
             # TODO to_csv drops column name
             assert_frame_equal(frame, df, check_names=False)
-            self.assertEqual(frame.index.names, df.index.names)
+            assert frame.index.names == df.index.names
 
             # needed if setUP becomes a classmethod
             self.frame.index = old_index
@@ -496,7 +494,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             # do not load index
             tsframe.to_csv(path)
             recons = DataFrame.from_csv(path, index_col=None)
-            self.assertEqual(len(recons.columns), len(tsframe.columns) + 2)
+            assert len(recons.columns) == len(tsframe.columns) + 2
 
             # no index
             tsframe.to_csv(path, index=False)
@@ -550,7 +548,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             df = _make_frame(True)
             df.to_csv(path, tupleize_cols=False, index=False)
             result = read_csv(path, header=[0, 1], tupleize_cols=False)
-            self.assertTrue(all([x is None for x in result.columns.names]))
+            assert all([x is None for x in result.columns.names])
             result.columns.names = df.columns.names
             assert_frame_equal(df, result)
 
@@ -587,15 +585,15 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             df = _make_frame(True)
             df.to_csv(path, tupleize_cols=False)
 
-            for i in [5, 6, 7]:
+            for i in [6, 7]:
                 msg = 'len of {i}, but only 5 lines in file'.format(i=i)
-                with assertRaisesRegexp(CParserError, msg):
+                with tm.assert_raises_regex(ParserError, msg):
                     read_csv(path, tupleize_cols=False,
                              header=lrange(i), index_col=0)
 
             # write with cols
-            with assertRaisesRegexp(TypeError, 'cannot specify cols with a '
-                                    'MultiIndex'):
+            with tm.assert_raises_regex(TypeError, 'cannot specify cols '
+                                        'with a MultiIndex'):
                 df.to_csv(path, tupleize_cols=False, columns=['foo', 'bar'])
 
         with ensure_clean('__tmp_to_csv_multiindex__') as path:
@@ -605,8 +603,8 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             exp = tsframe[:0]
             exp.index = []
 
-            self.assert_index_equal(recons.columns, exp.columns)
-            self.assertEqual(len(recons), 0)
+            tm.assert_index_equal(recons.columns, exp.columns)
+            assert len(recons) == 0
 
     def test_to_csv_float32_nanrep(self):
         df = DataFrame(np.random.randn(1, 4).astype(np.float32))
@@ -617,7 +615,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 
             with open(path) as f:
                 lines = f.readlines()
-                self.assertEqual(lines[1].split(',')[2], '999')
+                assert lines[1].split(',')[2] == '999'
 
     def test_to_csv_withcommas(self):
 
@@ -646,10 +644,10 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
                           index=df_float.index, columns=create_cols('date'))
 
         # add in some nans
-        df_float.ix[30:50, 1:3] = np.nan
+        df_float.loc[30:50, 1:3] = np.nan
 
         # ## this is a bug in read_csv right now ####
-        # df_dt.ix[30:50,1:3] = np.nan
+        # df_dt.loc[30:50,1:3] = np.nan
 
         df = pd.concat([df_float, df_int, df_bool, df_object, df_dt], axis=1)
 
@@ -815,7 +813,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
                     '2,"bar"\n'
                     '3,"baz"\n')
 
-        self.assertEqual(result, expected)
+        assert result == expected
 
     def test_to_csv_quote_none(self):
         # GH4328
@@ -826,7 +824,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
                       encoding=encoding, index=False)
             result = buf.getvalue()
             expected = 'A\nhello\n{"hello"}\n'
-            self.assertEqual(result, expected)
+            assert result == expected
 
     def test_to_csv_index_no_leading_comma(self):
         df = DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]},
@@ -838,7 +836,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
                     'one,1,4\n'
                     'two,2,5\n'
                     'three,3,6\n')
-        self.assertEqual(buf.getvalue(), expected)
+        assert buf.getvalue() == expected
 
     def test_to_csv_line_terminators(self):
         df = DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]},
@@ -850,7 +848,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
                     'one,1,4\r\n'
                     'two,2,5\r\n'
                     'three,3,6\r\n')
-        self.assertEqual(buf.getvalue(), expected)
+        assert buf.getvalue() == expected
 
         buf = StringIO()
         df.to_csv(buf)  # The default line terminator remains \n
@@ -858,7 +856,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
                     'one,1,4\n'
                     'two,2,5\n'
                     'three,3,6\n')
-        self.assertEqual(buf.getvalue(), expected)
+        assert buf.getvalue() == expected
 
     def test_to_csv_from_csv_categorical(self):
 
@@ -870,7 +868,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
         s.to_csv(res)
         exp = StringIO()
         s2.to_csv(exp)
-        self.assertEqual(res.getvalue(), exp.getvalue())
+        assert res.getvalue() == exp.getvalue()
 
         df = DataFrame({"s": s})
         df2 = DataFrame({"s": s2})
@@ -878,14 +876,14 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
         df.to_csv(res)
         exp = StringIO()
         df2.to_csv(exp)
-        self.assertEqual(res.getvalue(), exp.getvalue())
+        assert res.getvalue() == exp.getvalue()
 
     def test_to_csv_path_is_none(self):
         # GH 8215
         # Make sure we return string for consistency with
         # Series.to_csv()
         csv_str = self.frame.to_csv(path_or_buf=None)
-        self.assertIsInstance(csv_str, str)
+        assert isinstance(csv_str, str)
         recons = pd.read_csv(StringIO(csv_str), index_col=0)
         assert_frame_equal(self.frame, recons)
 
@@ -910,7 +908,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             text = f.read().decode('utf8')
             f.close()
             for col in df.columns:
-                self.assertIn(col, text)
+                assert col in text
 
     def test_to_csv_compression_bz2(self):
         # GH7615
@@ -933,7 +931,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
             text = f.read().decode('utf8')
             f.close()
             for col in df.columns:
-                self.assertIn(col, text)
+                assert col in text
 
     def test_to_csv_compression_xz(self):
         # GH11852
@@ -967,8 +965,8 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
         with ensure_clean() as filename:
             # zip compression is not supported and should raise ValueError
             import zipfile
-            self.assertRaises(zipfile.BadZipfile, df.to_csv,
-                              filename, compression="zip")
+            pytest.raises(zipfile.BadZipfile, df.to_csv,
+                          filename, compression="zip")
 
     def test_to_csv_date_format(self):
         with ensure_clean('__tmp_to_csv_date_format__') as path:
@@ -1080,13 +1078,13 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 1,False,3.2,,"b,c"
 """
         result = df.to_csv()
-        self.assertEqual(result, expected)
+        assert result == expected
 
         result = df.to_csv(quoting=None)
-        self.assertEqual(result, expected)
+        assert result == expected
 
         result = df.to_csv(quoting=csv.QUOTE_MINIMAL)
-        self.assertEqual(result, expected)
+        assert result == expected
 
         expected = """\
 "","c_bool","c_float","c_int","c_string"
@@ -1094,7 +1092,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 "1","False","3.2","","b,c"
 """
         result = df.to_csv(quoting=csv.QUOTE_ALL)
-        self.assertEqual(result, expected)
+        assert result == expected
 
         # see gh-12922, gh-13259: make sure changes to
         # the formatters do not break this behaviour
@@ -1104,14 +1102,14 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 1,False,3.2,"","b,c"
 """
         result = df.to_csv(quoting=csv.QUOTE_NONNUMERIC)
-        self.assertEqual(result, expected)
+        assert result == expected
 
         msg = "need to escape, but no escapechar set"
-        tm.assertRaisesRegexp(csv.Error, msg, df.to_csv,
-                              quoting=csv.QUOTE_NONE)
-        tm.assertRaisesRegexp(csv.Error, msg, df.to_csv,
-                              quoting=csv.QUOTE_NONE,
-                              escapechar=None)
+        tm.assert_raises_regex(csv.Error, msg, df.to_csv,
+                               quoting=csv.QUOTE_NONE)
+        tm.assert_raises_regex(csv.Error, msg, df.to_csv,
+                               quoting=csv.QUOTE_NONE,
+                               escapechar=None)
 
         expected = """\
 ,c_bool,c_float,c_int,c_string
@@ -1120,7 +1118,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 """
         result = df.to_csv(quoting=csv.QUOTE_NONE,
                            escapechar='!')
-        self.assertEqual(result, expected)
+        assert result == expected
 
         expected = """\
 ,c_bool,c_ffloat,c_int,c_string
@@ -1129,7 +1127,7 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
 """
         result = df.to_csv(quoting=csv.QUOTE_NONE,
                            escapechar='f')
-        self.assertEqual(result, expected)
+        assert result == expected
 
         # see gh-3503: quoting Windows line terminators
         # presents with encoding?
@@ -1137,17 +1135,39 @@ class TestDataFrameToCSV(tm.TestCase, TestData):
         df = pd.read_csv(StringIO(text))
         buf = StringIO()
         df.to_csv(buf, encoding='utf-8', index=False)
-        self.assertEqual(buf.getvalue(), text)
+        assert buf.getvalue() == text
 
         # xref gh-7791: make sure the quoting parameter is passed through
         # with multi-indexes
         df = pd.DataFrame({'a': [1, 2], 'b': [3, 4], 'c': [5, 6]})
         df = df.set_index(['a', 'b'])
         expected = '"a","b","c"\n"1","3","5"\n"2","4","6"\n'
-        self.assertEqual(df.to_csv(quoting=csv.QUOTE_ALL), expected)
+        assert df.to_csv(quoting=csv.QUOTE_ALL) == expected
 
+    def test_period_index_date_overflow(self):
+        # see gh-15982
 
-if __name__ == '__main__':
-    import nose
-    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
-                   exit=False)
+        dates = ["1990-01-01", "2000-01-01", "3005-01-01"]
+        index = pd.PeriodIndex(dates, freq="D")
+
+        df = pd.DataFrame([4, 5, 6], index=index)
+        result = df.to_csv()
+
+        expected = ',0\n1990-01-01,4\n2000-01-01,5\n3005-01-01,6\n'
+        assert result == expected
+
+        date_format = "%m-%d-%Y"
+        result = df.to_csv(date_format=date_format)
+
+        expected = ',0\n01-01-1990,4\n01-01-2000,5\n01-01-3005,6\n'
+        assert result == expected
+
+        # Overflow with pd.NaT
+        dates = ["1990-01-01", pd.NaT, "3005-01-01"]
+        index = pd.PeriodIndex(dates, freq="D")
+
+        df = pd.DataFrame([4, 5, 6], index=index)
+        result = df.to_csv()
+
+        expected = ',0\n1990-01-01,4\n,5\n3005-01-01,6\n'
+        assert result == expected
